"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setCookie = void 0;
const addDays_1 = require("../date/addDays");
const gxdate_1 = require("../types/gxdate");
const gxdatetime_1 = require("../types/gxdatetime");
const getCookie_1 = require("./getCookie");
/**
 * Sets a cookie
 * @param {string} name
 * @param {string} value
 * @param {string} path
 * @param {GxDate} expiration
 * @param {string} domain
 * @param {number} secure
 * @return number
 */
const setCookie = (name, value, path, expiration, domain, secure) => {
    path = path ? `;path=${path};` : "";
    if (expiration instanceof gxdate_1.GxDate) {
        expiration = expiration || (0, addDays_1.addDays)(new gxdate_1.GxDate(), 1);
    }
    else if (expiration instanceof gxdatetime_1.GxDatetime) {
        expiration = expiration || (0, addDays_1.addDays)(new gxdatetime_1.GxDatetime(), 1);
    }
    else {
        expiration = (0, addDays_1.addDays)(new gxdatetime_1.GxDatetime(), 1);
    }
    document.cookie = `${name}=${escape(value)};expires=${expiration.toUTCString()}${path}`;
    return (0, getCookie_1.getCookie)(name) === value ? 1 : 0;
};
exports.setCookie = setCookie;
//# sourceMappingURL=setCookie.js.map